==> eg_particle_interactions-v0.8.1-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/particle/spark/SparkFlash.java <==
package games.enchanted.eg_particle_interactions.common.particle.spark;

import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_3999;
import net.minecraft.class_4002;
import net.minecraft.class_4003;
import net.minecraft.class_4184;
import net.minecraft.class_4588;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import net.minecraft.client.particle.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SparkFlash extends class_4003 {
    private final class_4002 sprites;
    private final float originalQuadSize;
    protected int prevAge;
    protected final boolean useRandomAnimation;

    SparkFlash(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 sprites, boolean useRandomAnimation) {
        super(level, x, y, z, xSpeed, ySpeed, zSpeed);
        this.field_28787 = true;
        this.field_28786 = 0.96F;

        this.field_3852 = (xSpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.field_3869 = (ySpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.field_3850 = (zSpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.useRandomAnimation = useRandomAnimation;
        if(useRandomAnimation) {
            int rot = this.field_3840.method_39332(0, 3);
            this.field_3839 = rot * 90;
            this.field_3857 = field_3839;
        }

        this.field_3847 = this.field_3840.method_43048(4) + 3;

        this.sprites = sprites;
        this.method_18142(sprites);

        this.field_17867 = 2/16f;
        originalQuadSize = field_17867;
        prevAge = field_3866;
    }

    @Override
    public void method_3070() {
        prevAge = field_3866;
        super.method_3070();
        if(useRandomAnimation) {
            this.method_18140(this.sprites);
        } else {
            this.method_18142(this.sprites);
        }
    }

    @Override
    public int method_3068(float f) {
        return 240;
    }

    @Override
    public void method_3074(@NotNull class_4588 buffer, @NotNull class_4184 renderInfo, float partialTicks) {
        this.field_17867 = this.originalQuadSize * (0.5f + (Math.abs(1 - class_3532.method_16439(partialTicks, this.prevAge, this.field_3866) / this.field_3847) * 0.5f));
        super.method_3074(buffer, renderInfo, partialTicks);
    }

    @Override
    public @NotNull class_3999 method_18122() {
        return class_3999.field_17828;
    }

    public static class Provider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public Provider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            return new SparkFlash(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, false);
        }
    }

    public static class RandomAnimationProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public RandomAnimationProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            return new SparkFlash(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, true);
        }
    }
}

==> eg_particle_interactions-v0.8.4-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/particle/spark/SparkFlash.java <==
package games.enchanted.eg_particle_interactions.common.particle.spark;

import games.enchanted.eg_particle_interactions.common.particle.compat.CustomGeometryParticle;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_4002;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import org.jetbrains.annotations.Nullable;

public class SparkFlash extends CustomGeometryParticle {
    private final class_4002 sprites;
    private final float originalQuadSize;
    protected int prevAge;
    protected final boolean useRandomAnimation;

    SparkFlash(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 sprites, boolean useRandomAnimation) {
        super(level, x, y, z, xSpeed, ySpeed, zSpeed, sprites.method_18138(0, 1));
        this.field_28787 = true;
        this.field_28786 = 0.96F;

        this.field_3852 = (xSpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.field_3869 = (ySpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.field_3850 = (zSpeed / 2) + (Math.random() * 3.0 - 1.5) * 0.07 * (this.field_3840.method_43057() > 0.95 ? 2 : 1);
        this.useRandomAnimation = useRandomAnimation;
        if(useRandomAnimation) {
            int rot = this.field_3840.method_39332(0, 3);
            this.field_3839 = rot * 90;
            this.field_3857 = field_3839;
        }

        this.field_3847 = this.field_3840.method_43048(4) + 3;

        this.sprites = sprites;
        this.setSpriteFromAge(sprites);

        this.setScale(2/16f);
        originalQuadSize = this.getScale();
        prevAge = field_3866;
    }

    @Override
    public void method_3070() {
        prevAge = field_3866;
        super.method_3070();
        if(useRandomAnimation) {
            this.setSprite(this.sprites.method_18139(this.field_3840));
        } else {
            this.setSpriteFromAge(this.sprites);
        }
    }

    @Override
    public int method_3068(float f) {
        return 240;
    }

    @Override
    protected void renderTick(float partialTicks) {
        this.setScale(this.originalQuadSize * (0.5f + (Math.abs(1 - class_3532.method_16439(partialTicks, this.prevAge, this.field_3866) / this.field_3847) * 0.5f)));
    }

    @Override
    protected ParticleLayer getParticleLayer() {
        return ParticleLayer.OPAQUE;
    }

    public static class Provider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public Provider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            /*, RandomSource random
            *///?}
        ) {
            return new SparkFlash(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, false);
        }
    }

    public static class RandomAnimationProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public RandomAnimationProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            /*, RandomSource random
            *///?}
        ) {
            return new SparkFlash(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, true);
        }
    }
}
